// Distributed under GNU General Public License (see license.txt for details).
//
//  Copyright (c) 2006 Shai Shalev-Shwartz and Yoram Singer.
//  All Rights Reserved.
//
// Implementation of a support set.
// Assumes a fixed size for the support and a variable size for the alpha's.
//
// Uses BLAS/ATLAS or an equivalent vector and matrix infrastructure.
//

#ifndef KRANKER_SUPPORT_SET_H__
#define KRANKER_SUPPORT_SET_H__

#include <ext/hash_map>
#include "infra/infra.h"

using namespace __gnu_cxx;

class SupportSet {

 public:

  // The construction of a support-set initializes the support matrix and
  // currently never grows it.
  SupportSet(int max_support, int dimension) :
    support_(max_support, dimension), support_count_(0),
    inverse_index_(max_support) {
  }

  // Initialize the support-set from a file. Used in prediction mode.
  SupportSet(FILE * fd);

  // Return a "view" -- the actual support.
  infra::matrix_view operator() ();

  // Insert an instance to the support
  void Insert(infra::vector_view &instance, int ind);

  // Remove the instance whose index is ind from the support
  void Remove(int ind);

  // Retrieve the row whose index is ind from the support set.
  infra::vector_view Get(int ind);

  // Check whether an example is in the support
  bool Exist(int ind);

  // Return the actual size of the supprt set.
  int Size() {
    return support_count_;
  }

  // Save the support-set to a file.
  void Save(FILE *fd);

  // Save the support-set from a file.
  void Load(FILE *fd);

  ~SupportSet() { }

 protected:

  // Matrix of support patterns
  infra::matrix support_;

  // Number of support vectors.
  int support_count_;

  // The location of each example in the support.
  hash_map<int,int> index_;

  // The original index of each example in the support
  std::vector<int> inverse_index_;
};

#endif  // KRANKER_SUPPORT_SET_H__
