// Distributed under GNU General Public License (see license.txt for details).
//
//  Copyright (c) 2006 Shai Shalev-Shwartz and Yoram Singer.
//  All Rights Reserved.
//
// A simple converter from a text file to a file in infra format
//

#include "infra/infra.h"
#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>

using std::cerr;
using std::endl;
using std::string;

void error_read(char *s) {
  cerr << "Error when reading " << s << "\n";
  exit(EXIT_FAILURE);
}


int main(int argc, char *argv[])
{
  std::string infra_filename;
  std::string txt_filename;

  if (argc != 3) {
    cerr << "USAGE:\n\t" << argv[0] <<
      " txt_filename infra_filename\n";
    std::cerr << "\nDESCRIPTION:\n" <<
      "\tConvert txt_filename into infra format." <<
      "Output is written to infra_filename.\n" <<
      "\ttxt_filename is an ascii file formatted as follows:\n" <<
      "\tFirst two numbers in the file are m (num. of instances) " <<
      " and n (dimension)\n" <<
      "\tThen, m rows of the instances follows, each of which with n numbers\n";
    exit(EXIT_FAILURE);
  }


  // open input text file
  std::ifstream instream;
  instream.open(argv[1]);
  if (!instream.good()) {
    cerr << "Cannot open input file " << argv[1]  << "." << endl;
    return EXIT_FAILURE;
  }

  // open output infra file
  FILE *outstream;
  if ( (outstream=fopen(argv[2], "wb")) == NULL) {
    cerr << "Cannot open output file " << argv[2]  << "." << endl;
    return EXIT_FAILURE;
  }

  // read m and n
  unsigned long m,n;
  instream >> m; if (!instream.good()) error_read(argv[1]);
  instream >> n;

  // write m and n to the infra file
  infra::save_binary(outstream, m);
  infra::save_binary(outstream, n);

  // allocate vector
  infra::vector v(n);

  // loop
  for (unsigned int i=0; i<m;++i) {
    for (unsigned int j=0; j<n; ++j) {
      if (!instream.good()) error_read(argv[1]);
      instream >> v(j);
    }
    infra::save_binary(outstream, v);
  }

  // close files
  instream.close();
  fclose(outstream);

  return EXIT_SUCCESS;
}
